/*
 *
 * g_kde_dlg.cpp - part of FTE which is Copyright (c) 1994-1996, Marko Macek

 * KDE Gui implementation for the FTE Editor
 * derived from the original Qt GUI module written by Marko Macek
 *
 * this file was initially written by Roberto Alsina
 *
 * GUI modifications (to be more KDE conform) by Alexander Sanda - as@psa.at
 *
 *    You may distribute under the terms of either the GNU General Public
 *    License or the Artistic License, as specified in the README file.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include <stdarg.h>

#include "fte.h"
#include "feature.h"
#include "s_files.h"
#include "c_hilit.h"
#include "e_buffer.h"
#include "c_history.h"

#include <qregexp.h>
#include <qbutton.h>
#include <qchkbox.h>
#include <qframe.h>
#include <qfiledlg.h>
#include <qlayout.h>
#include <qlined.h>
#include <qpushbt.h>
#include <qwidget.h>
#include <qgrid.h>
#include <qvbox.h>

#include <kfiledialog.h>
#include <qfontmetrics.h>

#include "g_kde_dlg.h"

void cleanForRichText(QString &text)
{
    text.replace(QRegExp("<"),"&lt;");
    text.replace(QRegExp(">"),"&gt;");
}


unsigned long HaveGUIDialogs = GUIDLG_FILE | GUIDLG_CHOICE | GUIDLG_PROMPT | GUIDLG_FIND | GUIDLG_FINDREPLACE;

PromptDlg::PromptDlg
    () : QDialog(0,0, TRUE )
{

    QVBoxLayout *vbl=new QVBoxLayout (this);

    QVBox *l1=new QVBox (this);
    vbl->addWidget(l1);
    vbl->activate();

    label=new QLabel (l1, "_box");
    entry=new KLineEdit ("",l1);

    QHBox *l2=new QHBox(l1);

    QPushButton *b1=new QPushButton(l2);
    QPushButton *b2=new QPushButton(l2);

    b1->setText("Ok");
    b1->setDefault( true );
    b2->setText("Cancel");

    QObject::connect (b1,SIGNAL(clicked()),SLOT(accept()));
    QObject::connect (b2,SIGNAL(clicked()),SLOT(reject()));
    entry->setFocus();

    b1->setMinimumSize(b1->sizeHint());
    b2->setMinimumSize(b2->sizeHint());

    if (b1->minimumSize().width() <80)
        b1->setFixedWidth(80);
    if (b2->minimumSize().width() <80)
        b2->setFixedWidth(80);


    resize(300,110);
    b1->move(10, height() - 30);
    b2->move(width() - 90, height() - 30);
}

FindDlg::FindDlg
    (long options) : QDialog(0,0, TRUE )
{

    QVBoxLayout *vbl=new QVBoxLayout (this);

    QVBox *vbox=new QVBox (this);
    vbl->addWidget(vbox);
    vbl->activate();


    new QLabel ("Search for:", vbox);
    entry=new KLineEdit ("", vbox);

    QGrid *soframe = new QGrid(2,vbox, "_frame");

    QHBox *hbox=new QHBox(vbox);

    QPushButton *b1=new QPushButton(hbox);
    QPushButton *b2=new QPushButton(hbox);
    b1->setText("Find");
    b1->setDefault( true );
    b2->setText("Cancel");
    QObject::connect (b1,SIGNAL(clicked()),SLOT(accept()));
    QObject::connect (b2,SIGNAL(clicked()),SLOT(reject()));
    entry->setFocus();

    a=new QCheckBox("&All Occurences", soframe);
    a->setChecked(options & SEARCH_ALL);
    b=new QCheckBox("In &Block", soframe);
    b->setChecked(options & SEARCH_BLOCK);
    c=new QCheckBox("&Center finds", soframe);
    c->setChecked(options & SEARCH_CENTER);
    d=new QCheckBox("&Delete Line", soframe);
    d->setChecked(options & SEARCH_DELETE);
    e=new QCheckBox("&e option??", soframe);
    e->setChecked(false);
    g=new QCheckBox("&Global Search", soframe);
    g->setChecked(options & SEARCH_GLOBAL);
    i=new QCheckBox("&Ignore Case", soframe);
    i->setChecked(options & SEARCH_NCASE);
    j=new QCheckBox("&Join Line", soframe);
    j->setChecked(options & SEARCH_JOIN);
    r=new QCheckBox("&Reverse search", soframe);
    r->setChecked(options & SEARCH_BACK);
    x=new QCheckBox("Use RegE&xp", soframe);
    x->setChecked(options & SEARCH_RE);


    a->setMinimumSize(a->sizeHint());
    b->setMinimumSize(b->sizeHint());
    c->setMinimumSize(c->sizeHint());
    d->setMinimumSize(d->sizeHint());
    e->setMinimumSize(e->sizeHint());
    g->setMinimumSize(g->sizeHint());
    i->setMinimumSize(i->sizeHint());
    j->setMinimumSize(j->sizeHint());
    r->setMinimumSize(r->sizeHint());
    x->setMinimumSize(x->sizeHint());

    // button geometry

    b1->setFixedSize(b1->sizeHint());
    b2->setFixedSize(b2->sizeHint());
    if (b1->minimumSize().width() <80)
        b1->setFixedWidth(80);
    if (b2->minimumSize().width() <80)
        b2->setFixedWidth(80);

    setCaption("Search for text");
}

FindReplaceDlg::FindReplaceDlg
    (long options) : QDialog(0,0, TRUE )
{

    QVBoxLayout *vbl=new QVBoxLayout (this);

    QVBox *vbox=new QVBox (this);
    vbl->addWidget(vbox);
    vbl->activate();

    new QLabel ("Search for:",vbox);
    entry=new KLineEdit ("", vbox);
    new QLabel ("Replace with",vbox);
    entry2=new KLineEdit ("", vbox);

    QGrid *soframe = new QGrid(2,vbox, "_frame");

    QHBox *hbox=new QHBox(vbox);

    QPushButton *b1=new QPushButton(hbox);
    QPushButton *b2=new QPushButton(hbox);
    b1->setText("Ok");
    b1->setDefault( true );
    b2->setText("Cancel");

    QObject::connect (b1,SIGNAL(clicked()),SLOT(accept()));
    QObject::connect (b2,SIGNAL(clicked()),SLOT(reject()));
    entry->setFocus();

    a=new QCheckBox("&All Occurences", soframe);
    a->setChecked(options & SEARCH_ALL);
    b=new QCheckBox("In &Block", soframe);
    b->setChecked(options & SEARCH_BLOCK);
    c=new QCheckBox("&Center finds", soframe);
    c->setChecked(options & SEARCH_CENTER);
    d=new QCheckBox("&Delete Line", soframe);
    d->setChecked(options & SEARCH_DELETE);
    e=new QCheckBox("&e option??", soframe);
    e->setChecked(false);
    g=new QCheckBox("&Global Search", soframe);
    g->setChecked(options & SEARCH_GLOBAL);
    i=new QCheckBox("&Ignore Case", soframe);
    i->setChecked(options & SEARCH_NCASE);
    j=new QCheckBox("&Join Line", soframe);
    j->setChecked(options & SEARCH_JOIN);
    r=new QCheckBox("&Reverse search", soframe);
    r->setChecked(options & SEARCH_BACK);
    x=new QCheckBox("Use RegE&xp", soframe);
    x->setChecked(options & SEARCH_RE);

    a->setMinimumSize(a->sizeHint());
    b->setMinimumSize(b->sizeHint());
    c->setMinimumSize(c->sizeHint());
    d->setMinimumSize(d->sizeHint());
    e->setMinimumSize(e->sizeHint());
    g->setMinimumSize(g->sizeHint());
    i->setMinimumSize(i->sizeHint());
    j->setMinimumSize(j->sizeHint());
    r->setMinimumSize(r->sizeHint());
    x->setMinimumSize(x->sizeHint());

    // button geometry

    b1->setFixedSize(b1->sizeHint());
    b2->setFixedSize(b2->sizeHint());
    if (b1->minimumSize().width() <80)
        b1->setFixedWidth(80);
    if (b2->minimumSize().width() <80)
        b2->setFixedWidth(80);

    setCaption("Search and replace");
}

int DLGGetFile(GView *v, char const *Prompt, unsigned int BufLen, char *FileName, int) {

    QString fn;
    char filter[MAXPATH] = "*";
    char directory[MAXPATH];

    assert(BufLen > 0);

    v = v;

    JustDirectory(FileName, directory);

    fn=KFileDialog::getOpenFileName(QString::fromLatin1(directory),QString::fromLatin1(filter));

    if (fn.isNull())
        return 0;
    strncpy(FileName, fn, BufLen);
    FileName[BufLen - 1] = 0;
    DEBUGX(("selected %s\n", FileName));
    return FileName[0] ? 1 : 0;
}

QChoiceBox::QChoiceBox(QWidget *parent, const char *name)
    : QDialog(parent, name, TRUE)
{

    buttonSelected = -1;
    buttonCount = 0;
    buttonActivated = -1;
    buttonArmed = -1;

    label = new QLabel(this, "text");
    CHECK_PTR(label);
}

void QChoiceBox::setText(const char *_text) {
    QString text(_text);
    cleanForRichText(text);
    QString t("<center>");
    t.append(text);
    if(!strcmp(caption(), "About")) {
        t.append("<br>");
        t.append("KDE(tm) additions by Roberto Alsina");
    }
    t.append("</center>");
    label->setText(t);
    adjustSize();
}

void QChoiceBox::addButton(const char *text) {
    assert(buttonCount < kMaxButton);

    button[buttonCount] = new QPushButton(this);
    CHECK_PTR(button[buttonCount]);
    connect(button[buttonCount], SIGNAL(clicked()), SLOT(clicked()));
    connect(button[buttonCount], SIGNAL(pressed()), SLOT(pressed()));
    connect(button[buttonCount], SIGNAL(released()), SLOT(released()));
    button[buttonCount]->setText(text ? text : "?" );
    button[buttonCount]->resize(100, button[buttonCount]->fontMetrics().height()+10);
    buttonCount++;
}

void QChoiceBox::adjustSize() {
    int w_buttons = buttonCount * 100 + (buttonCount - 1) * 10;

    label->adjustSize();

    int w = QMAX(w_buttons, label->width());
    int h = button[0]->height() + label->height();
    h += 40;  // margins
    w += 20;  // margins
    resize( w, h);
    for(int i = 0; i < buttonCount; i++)
        button[i]->move((width() - w_buttons) / 2 + (i * 100) + (i * 10), height() - 30);
    setMinimumSize(w, h);
    setMaximumSize(w, h);
    label->move((w - label->width()) / 2, 10);
}

/*
void QChoiceBox::resizeEvent( QResizeEvent * ) {

    adjustSize();
    }
    */

void QChoiceBox::pressed() {
    int i;

    buttonSelected = -1;
    for (i = 0; i < buttonCount; i++)
        if (button[i]->isDown())
            buttonSelected = i;
    buttonArmed = buttonSelected;
    DEBUGX(("selected: %d\n", buttonSelected));
}

void QChoiceBox::released() {
    buttonSelected = -1;
    DEBUGX(("released\n"));
}

void QChoiceBox::clicked() {
    buttonActivated = buttonArmed;
    DEBUGX(("activated: %d\n", buttonActivated));
    accept();
}

int DLGPickChoice(GView *, const char *ATitle, int NSel, va_list ap, int)
{
    QChoiceBox *cb = new QChoiceBox();
    CHECK_PTR(cb);
    cb->setCaption( ATitle );

    for (int i = 0; i < NSel; i++)
        cb->addButton(va_arg(ap, char *));

    char msg[1024];
    char *fmt;
    fmt = va_arg(ap, char *);
    vsprintf(msg, fmt, ap);

    cb->setText(msg);

    int retcode = cb->exec();

    if (retcode == QDialog::Accepted)
    {
       int tmp=cb->getChoice();
       delete cb;
       return tmp;
    };
    delete cb;
    return -1;
}


void InsertCompletionHistory(KCompletion *comp, int id, int maxlen) {
    int i, count;
    char *str;
    count = CountInputHistory(id);

    str = (char *)malloc(maxlen + 1);
    if (str == 0)
        return;

    for (i = 0; i < count; i++) {
        if (GetInputHistory(id, str, maxlen, i + 1) == 1)
            comp->addItem(str);
    }
    free(str);
}

//static SearchReplaceOptions SearchOpt;


int DLGGetFind(GView *, SearchReplaceOptions &sr) {

    FindDlg *dlg=new FindDlg(sr.Options);
    KCompletion *comp=dlg->entry->completionObject();
    InsertCompletionHistory (comp,HIST_SEARCH,50);
    int retcode= dlg->exec();

    sr.ok=0;

    if (retcode==QDialog::Accepted)
    {
        sr.ok=1;
        strncpy(sr.strSearch, dlg->entry->text(), MAXSEARCH);
        sr.strSearch[MAXSEARCH - 1] = 0;
        AddInputHistory (HIST_SEARCH,sr.strSearch);

        sr.Options=0;
        sr.Options+=dlg->a->isChecked()*SEARCH_ALL;
        sr.Options+=dlg->b->isChecked()*SEARCH_BLOCK;
        sr.Options+=dlg->c->isChecked()*SEARCH_CENTER;
        sr.Options+=dlg->d->isChecked()*SEARCH_DELETE;
        sr.Options+=dlg->e->isChecked()*0;
        sr.Options+=dlg->g->isChecked()*SEARCH_GLOBAL;
        sr.Options+=dlg->i->isChecked()*SEARCH_NCASE;
        sr.Options+=dlg->j->isChecked()*SEARCH_JOIN;
        sr.Options+=dlg->r->isChecked()*SEARCH_BACK;
        sr.Options+=dlg->x->isChecked()*SEARCH_RE;

        return 1;
//        return sr.strSearch[0] ? 1 : 0;
    }
    else return 0;
}



int DLGGetFindReplace(GView *, SearchReplaceOptions &sr) {

    SearchOpt = sr;

    FindReplaceDlg *dlg=new FindReplaceDlg(sr.Options);
    KCompletion *comp=dlg->entry->completionObject();
    KCompletion *comp2=dlg->entry2->completionObject();

    InsertCompletionHistory (comp,HIST_SEARCH,50);
    InsertCompletionHistory (comp2,HIST_SEARCH,50);
    int retcode= dlg->exec();

    sr.ok=0;

    if (retcode==QDialog::Accepted)
    {
        sr.ok=1;
        strncpy(sr.strSearch, dlg->entry->text(), MAXSEARCH);
        sr.strSearch[MAXSEARCH - 1] = 0;
        AddInputHistory (HIST_SEARCH,sr.strSearch);

        strncpy(sr.strReplace, dlg->entry2->text(), MAXSEARCH);
        sr.strReplace[MAXSEARCH - 1] = 0;
        AddInputHistory (HIST_SEARCH,sr.strReplace);

        sr.Options=0;
        sr.Options+=dlg->a->isChecked()*SEARCH_ALL;
        sr.Options+=dlg->b->isChecked()*SEARCH_BLOCK;
        sr.Options+=dlg->c->isChecked()*SEARCH_CENTER;
        sr.Options+=dlg->d->isChecked()*SEARCH_DELETE;
        sr.Options+=dlg->e->isChecked()*0;
        sr.Options+=dlg->g->isChecked()*SEARCH_GLOBAL;
        sr.Options+=dlg->i->isChecked()*SEARCH_NCASE;
        sr.Options+=dlg->j->isChecked()*SEARCH_JOIN;
        sr.Options+=dlg->r->isChecked()*SEARCH_BACK;
        sr.Options+=dlg->x->isChecked()*SEARCH_RE;
        sr.Options|=SEARCH_REPLACE;

        return 1;
    }
    sr=SearchOpt;
    return 0;
}



int DLGGetStr(GView *, char const *Prompt, unsigned int BufLen, char *Str, int HistId, int) {

    QString caption;

    assert(BufLen > 0);

    caption = "KFte ";
    caption.append(Prompt);

    PromptDlg *dlg=new PromptDlg();
    dlg->label->setText(Prompt);
    dlg->setCaption(caption);

    KCompletion *comp=dlg->entry->completionObject();
    InsertCompletionHistory (comp,HistId,50);


    dlg->entry->setText(Str);
    dlg->entry->selectAll();
    int retcode= dlg->exec();

    if (retcode==QDialog::Accepted)
    {
        strncpy(Str, dlg->entry->text(), BufLen);
        Str[BufLen - 1] = 0;
        AddInputHistory (HistId,Str);
        return Str[0] ? 1 : 0;
    }
    else return 0;
}

#include "g_kde_dlg.moc"
